<?php

namespace BexioConnectorByHelvy;

/**
 * Handle backend scripts
 *
 * @since 0.1.0
 */

defined('ABSPATH') || exit;

class AdminPanelAssets
{
	public function __construct()
	{
		add_action('admin_enqueue_scripts', array(__CLASS__, 'admin_enqueue_scripts'), 10, 1);

		add_filter( 'manage_woocommerce_page_wc-orders_columns', array($this, 'add_order_admin_bexio_column') );
		add_action( 'woocommerce_shop_order_list_table_custom_column', array($this,'populate_order_admin_bexio_column'), 10, 2 );
		add_action( 'woocommerce_admin_order_data_after_order_details', array($this,'display_bexio_metadata_in_admin_order_details') );
	}
	
	/**
	 * Display Bexio metadata in WooCommerce order details.
	 * 
	 * @param \WC_Order $order WooCommerce order
	 */
	public function display_bexio_metadata_in_admin_order_details($order) {
		$bexio_meta = [
			'invoice_id' => $order->get_meta(AdminAPISettings::$prefix . 'bexio_invoice_id'),
			'invoice_number' => $order->get_meta(AdminAPISettings::$prefix . 'bexio_invoice_number'),
			'order_id' => $order->get_meta(AdminAPISettings::$prefix . 'bexio_order_id'),
			'order_number' => $order->get_meta(AdminAPISettings::$prefix . 'bexio_order_number'),
			'payment_id' => $order->get_meta(AdminAPISettings::$prefix . 'bexio_payment_id'),
			'fulfillment_id' => $order->get_meta(AdminAPISettings::$prefix . 'bexio_fulfillment_id'),
		];

		$order_html = $bexio_meta['order_id']
			? '<a href="https://office.bexio.com/index.php/kb_order/show/id/'.$bexio_meta['order_id'].'" target="_blank">' . $bexio_meta['order_number'] . '</a>'
			: '/';

		$invoice_html = $bexio_meta['invoice_id']
			? '<a href="https://office.bexio.com/index.php/kb_invoice/show/id/'.$bexio_meta['invoice_id'].'" target="_blank">' . $bexio_meta['invoice_number'] . '</a>'
			: '/';
	?>
		<div style="position: relative;top: 12px;">
			<h4>Bexio Resources</h4>
			<p>
				<?php 
					echo '<strong>Order:</strong> ' . $order_html . '<br />';
					echo '<strong>Invoice:</strong> ' . $invoice_html;
				?>
			</p>
		</div>
	<?php
	}

	/**
	 * Populate bexio column in WooCommerce Orders list.
	 * 
	 * @param string $column Column name
	 * @param \WC_Order $order WooCommerce order
	 */
	public function populate_order_admin_bexio_column( $column, $order ) {
		if ( 'bexio' !== $column ) {
			return;
		}

		$bexio_meta = [
			'invoice_id' => $order->get_meta(AdminAPISettings::$prefix . 'bexio_invoice_id'),
			'invoice_number' => $order->get_meta(AdminAPISettings::$prefix . 'bexio_invoice_number'),
			'order_id' => $order->get_meta(AdminAPISettings::$prefix . 'bexio_order_id'),
			'order_number' => $order->get_meta(AdminAPISettings::$prefix . 'bexio_order_number'),
			'payment_id' => $order->get_meta(AdminAPISettings::$prefix . 'bexio_payment_id'),
			'fulfillment_id' => $order->get_meta(AdminAPISettings::$prefix . 'bexio_fulfillment_id'),
		];

		if (empty($bexio_meta['invoice_id']) && empty($bexio_meta['order_id'])) {
			echo '/';
			return;
		}
		
		$html = '';
		if ($bexio_meta['invoice_id']) {
			$html .= '<a href="https://office.bexio.com/index.php/kb_invoice/show/id/'.$bexio_meta['invoice_id'].'" target="_blank">' . $bexio_meta['invoice_number'] . '</a>';
		}
		if ($bexio_meta['order_id']) {
			$html .= "<br>" . '<a href="https://office.bexio.com/index.php/kb_order/show/id/'.$bexio_meta['order_id'].'" target="_blank">' . $bexio_meta['order_number'] . '</a>';
		}

		echo $html;
	}

	/**
	 * Add bexio column to WooCommerce Orders list.
	 *
	 * @param array $columns Columns.
	 * @return array
	 */
	public function add_order_admin_bexio_column( $columns ) {
		$new_columns = array();
		foreach ($columns as $column_name => $column_info) {
			$new_columns[$column_name] = $column_info;
			if ('order_total' === $column_name) {
				$new_columns['bexio'] = 'Bexio';
			}
		}
		return $new_columns;
	}

	/**
	 * Enqueue Backend Scripts
	 *
	 * @since 0.1.0
	 */
	public static function admin_enqueue_scripts()
	{
		$currentScreen = get_current_screen();
		$screenID = $currentScreen->id;

		if ($screenID === "woocommerce_page_bexio-connector-by-helvy") {
			$apiNonce = wp_create_nonce('wp_rest');
			$root = rest_url(BEXIO_CONNECTOR_BY_HELVY_REST_API_ROUTE . '/');
			$baseUrl = BEXIO_CONNECTOR_BY_HELVY_URL; // can be used for assets

			if (defined('BEXIO_CONNECTOR_BY_HELVY_ENV') && BEXIO_CONNECTOR_BY_HELVY_ENV == "dev") {
				include_once plugin_dir_path( __FILE__ ) . "../../includes_dev/enqueue_front.php";
			} else {
				wp_enqueue_script('bexio-connector-by-helvy-backend', BEXIO_CONNECTOR_BY_HELVY_URL . 'assets/js/index.js', array('wp-i18n'), BEXIO_CONNECTOR_BY_HELVY_VERSION, true);
				wp_localize_script(
					'bexio-connector-by-helvy-backend',
					'bexioConnectorByHelvy',
					array(
						'apiNonce'     => $apiNonce,
						'root'          => $root,
						'baseUrl'   => $baseUrl,
						'adminUrl' => admin_url(),
						'userLocale' => get_user_locale(),
					)
				);
			}
		}
	}
}
