<?php

namespace BexioConnectorByHelvy;

/**
 * Handle WooCommerce Webhooks Events
 *
 * @since 0.1.0
 */

defined('ABSPATH') || exit;

class WooCommerceWebhooks
{

	public function __construct()
	{
        add_action('woocommerce_new_order', array($this, 'handle_order_created'));
        add_action('woocommerce_order_status_processing', array($this, 'handle_order_paid'));
        add_action('woocommerce_order_status_completed', array($this, 'handle_order_completed'));
	}

    /**
     * Triggered when a new order is created
     * @param $order_id int Order ID
     * @return void
     */
	function handle_order_created($order_id) {
        $order = wc_get_order($order_id);
        
        // Skip processing if the order is in a non-syncable state
        $order_status = $order->get_status();
        if (in_array($order_status, array('draft', 'cancelled', 'refunded', 'failed'))) {
            return;
        }

        WCToBexio::instance()->sync_order($order);
        WCToBexio::instance()->sync_order_payment($order);
        WCToBexio::instance()->sync_order_fulfillment($order);
    }

    /**
     * Triggered when an order is paid
     * @param $order_id int Order ID
     * @return void
     */
	function handle_order_paid($order_id) {
        $order = wc_get_order($order_id);
        WCToBexio::instance()->sync_order_payment($order);
    }

    /**
     * Triggered when an order is completed
     * @param $order_id int Order ID
     * @return void
     */
	function handle_order_completed($order_id) {
        $order = wc_get_order($order_id);
        WCToBexio::instance()->sync_order_fulfillment($order);
    }
}
