<?php

namespace BexioConnectorByHelvy;

/**
 * Admin API Auth
 *
 * @since 0.1.0
 */

defined('ABSPATH') || exit;

class AdminAPIAuth
{

    public function __construct()
    {

        add_action('rest_api_init', function () {
            // Auth
            register_rest_route(BEXIO_CONNECTOR_BY_HELVY_REST_API_ROUTE, '/auth/register/', array(
                'methods' => 'POST',
                'callback' => array($this, 'register_customer'),
                'permission_callback' => array($this, 'get_permission')
            ));
            register_rest_route(BEXIO_CONNECTOR_BY_HELVY_REST_API_ROUTE, '/auth/bexio/', array(
                'methods' => 'GET',
                'callback' => array($this, 'connect_to_bexio'),
                'permission_callback' => array($this, 'get_permission')
            ));
        });
    }

    public function register_customer($data)
    {
        $siteDomain = Utils::extract_domain(get_site_url());
        $body = $data->get_params();
        $customerData = [
            "hostname" => $siteDomain,
            "adminUrl" => get_admin_url(),
            "address" => $body["address"],
            "city" =>  $body["city"],
            "company" =>  $body["company"],
            "country" =>  $body["country"],
            "email" =>  $body["email"],
            "name" =>  $body["name"],
            "vatNumber" =>  $body["vatNumber"],
            "zip" =>  $body["zip"],
        ];
        $apiRes = API::instance()->register_customer($customerData);

        if ($apiRes instanceof \WP_Error) {
            return new \WP_REST_Response($apiRes, 500);
        }

        update_option( 'bexio_connector_by_helvy_api_access_token', sanitize_text_field( $apiRes["tokens"]["access"]["token"] ) );
        update_option( 'bexio_connector_by_helvy_api_refresh_token', sanitize_text_field( $apiRes["tokens"]["refresh"]["token"] ) );
        update_option( 'bexio_connector_by_helvy_api_hash', sanitize_text_field( $apiRes["user"]["newPassword"] ) );
        
        return new \WP_REST_Response($apiRes, 200);
    }

    public function connect_to_bexio()
    {
        $apiRes = API::instance()->connect_to_bexio();
        if ($apiRes instanceof \WP_Error) {
            return new \WP_REST_Response($apiRes, 500);
        }

        return new \WP_REST_Response($apiRes, 200);
    }

    /**
     * Permission Callback
     **/
    public function get_permission()
    {
        if (current_user_can('administrator') || current_user_can('manage_woocommerce')) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Cloning is forbidden.
     *
     * @since 1.0.0
     */
    public function __clone()
    {
        _doing_it_wrong(__FUNCTION__, __('Cheatin&#8217; huh?'), BEXIO_CONNECTOR_BY_HELVY_VERSION);
    }
}
