<?php

namespace BexioConnectorByHelvy;

/**
 * Admin API Synchronization
 *
 * @since 0.1.0
 */

defined('ABSPATH') || exit;

class AdminAPISynchronization
{

    public function __construct()
    {
        add_action('rest_api_init', function () {
            // Usage quota
            register_rest_route(BEXIO_CONNECTOR_BY_HELVY_REST_API_ROUTE, '/sync/usage-quota', array(
                'methods' => 'GET',
                'callback' => array($this, 'get_usage_quota'),
                'permission_callback' => array($this, 'get_permission')
            ));
            // Orders sync state
            register_rest_route(BEXIO_CONNECTOR_BY_HELVY_REST_API_ROUTE, '/orders/(?P<page>\d+)', array(
                'methods' => 'GET',
                'callback' => array($this, 'get_orders'),
                'args' => array(
                    'page' => array(
                        'validate_callback' => function ($param, $request, $key) {
                            return is_numeric($param);
                        }
                    ),
                ),
                'permission_callback' => array($this, 'get_permission')
            ));
            // Sync order
            register_rest_route(BEXIO_CONNECTOR_BY_HELVY_REST_API_ROUTE, '/sync/order/(?P<order_id>\d+)', array(
                'methods' => 'POST',
                'callback' => array($this, 'sync_order'),
                'args' => array(
                    'order_id' => array(
                        'validate_callback' => function ($param, $request, $key) {
                            return is_numeric($param);
                        }
                    ),
                ),
                'permission_callback' => array($this, 'get_permission')
            ));
            register_rest_route(BEXIO_CONNECTOR_BY_HELVY_REST_API_ROUTE, '/sync/order/(?P<order_id>\d+)/payment', array(
                'methods' => 'POST',
                'callback' => array($this, 'sync_order_payment'),
                'args' => array(
                    'order_id' => array(
                        'validate_callback' => function ($param, $request, $key) {
                            return is_numeric($param);
                        }
                    ),
                ),
                'permission_callback' => array($this, 'get_permission')
            ));
            register_rest_route(BEXIO_CONNECTOR_BY_HELVY_REST_API_ROUTE, '/sync/order/(?P<order_id>\d+)/fulfillment', array(
                'methods' => 'POST',
                'callback' => array($this, 'sync_order_fulfillment'),
                'args' => array(
                    'order_id' => array(
                        'validate_callback' => function ($param, $request, $key) {
                            return is_numeric($param);
                        }
                    ),
                ),
                'permission_callback' => array($this, 'get_permission')
            ));
            // Delete metadata
            register_rest_route(BEXIO_CONNECTOR_BY_HELVY_REST_API_ROUTE, '/sync/order/(?P<order_id>\d+)/metadata', array(
                'methods' => 'DELETE',
                'callback' => array($this, 'delete_order_metadata'),
                'args' => array(
                    'order_id' => array(
                        'validate_callback' => function ($param, $request, $key) {
                            return is_numeric($param);
                        }
                    ),
                ),
                'permission_callback' => array($this, 'get_permission')
            ));
        });
    }

    /*
    * Fetching orders example.
    */
    public function get_orders($request)
    {
        $params = $request->get_params();

        if (isset($params['page']) && is_numeric($params['page'])) {
            $page = intval($params['page']);
            $per_page = 10;

            $args = array(
                'limit'        => $per_page,
                'page'         => $page,
                'orderby'      => 'date',
                'order'        => 'DESC',
            );

            // Fetch orders
            $orders = wc_get_orders($args);
            // Fetch total number of orders
            $total_orders = wc_orders_count('completed') + wc_orders_count('processing') + wc_orders_count('on-hold') + wc_orders_count('pending') + wc_orders_count('cancelled') + wc_orders_count('refunded') + wc_orders_count('failed');

            $order_data = array();

            foreach ($orders as $order) {
                if ($order) {
                    $formatted_date = gmdate('M d, Y', strtotime($order->get_date_created()));
                    $customer_name = $order->get_billing_first_name() . ' ' . $order->get_billing_last_name();

                    $order_data[] = array(
                        'id'       => $order->get_id(),
                        'date'     => $formatted_date,
                        'customer' => $customer_name,
                        'total'    => $order->get_total(),
                        'currency'    => $order->get_currency(),
                        'metaBexioInvoiceID'    => $order->get_meta(AdminAPISettings::$prefix . 'bexio_invoice_id'),
                        'metaBexioInvoiceNumber'    => $order->get_meta(AdminAPISettings::$prefix . 'bexio_invoice_number'),
                        'metaBexioOrderID'    => $order->get_meta(AdminAPISettings::$prefix . 'bexio_order_id'),
                        'metaBexioOrderNumber'    => $order->get_meta(AdminAPISettings::$prefix . 'bexio_order_number'),
                        'metaBexioPaymentID'    => $order->get_meta(AdminAPISettings::$prefix . 'bexio_payment_id'),
                        'metaBexioFulfillmentID'    => $order->get_meta(AdminAPISettings::$prefix . 'bexio_fulfillment_id'),
                        // 'orderStatus'   => $order->get_status(),
                    );
                }
            }

            $response = array(
                'total' => $total_orders,
                'data'  => $order_data,
            );

            return new \WP_REST_Response($response, 200);
        }

        return new \WP_REST_Response(array('message' => 'Something went wrong!'), 403);
    }

    /*
    * Retrieve the current usage quota
    */
    public function get_usage_quota($request)
    {
        // $params = $request->get_params();
        // if (!isset($params['order_id']) && !is_numeric($params['order_id'])) {
        //     new \WP_REST_Response(array('message' => 'Missing order ID'), 403);
        // }
        
        // $order_id = intval($params['order_id']);
        // $order = wc_get_order($order_id);
        // if (!$order) {
        //     new \WP_REST_Response(array('message' => 'Order not found'), 404);
        // }

        $res = API::instance()->get_usage_quota();
        return new \WP_REST_Response($res);
    }

    /*
    * Sync the given order to bexio.
    */
    public function sync_order($request)
    {
        $params = $request->get_params();
        if (!isset($params['order_id']) && !is_numeric($params['order_id'])) {
            new \WP_REST_Response(array('message' => 'Missing order ID'), 403);
        }
        
        $order_id = intval($params['order_id']);
        $order = wc_get_order($order_id);
        if (!$order) {
            new \WP_REST_Response(array('message' => 'Order not found'), 404);
        }

        $res = WCToBexio::instance()->sync_order($order, true);
        return new \WP_REST_Response($res);
    }

    /*
    * Sync the payment of the given order to bexio.
    */
    public function sync_order_payment($request)
    {
        $params = $request->get_params();
        if (!isset($params['order_id']) && !is_numeric($params['order_id'])) {
            new \WP_REST_Response(array('message' => 'Missing order ID'), 403);
        }
        
        $order_id = intval($params['order_id']);
        $order = wc_get_order($order_id);
        if (!$order) {
            new \WP_REST_Response(array('message' => 'Order not found'), 404);
        }

        $res = WCToBexio::instance()->sync_order_payment($order, true);
        return new \WP_REST_Response($res);
    }

    /*
    * Sync the fulfillment of the given order to bexio.
    */
    public function sync_order_fulfillment($request)
    {
        $params = $request->get_params();
        if (!isset($params['order_id']) && !is_numeric($params['order_id'])) {
            new \WP_REST_Response(array('message' => 'Missing order ID'), 403);
        }
        
        $order_id = intval($params['order_id']);
        $order = wc_get_order($order_id);
        if (!$order) {
            new \WP_REST_Response(array('message' => 'Order not found'), 404);
        }

        $res = WCToBexio::instance()->sync_order_fulfillment($order, true);
        return new \WP_REST_Response($res);
    }

    /*
    * Delete metadata of the given order.
    */
    public function delete_order_metadata($request)
    {
        $params = $request->get_params();
        if (!isset($params['order_id']) && !is_numeric($params['order_id'])) {
            new \WP_REST_Response(array('message' => 'Missing order ID'), 403);
        }
        
        $order_id = intval($params['order_id']);
        $order = wc_get_order($order_id);

        $order->delete_meta_data(AdminAPISettings::$prefix . 'bexio_order_id');
        $order->delete_meta_data(AdminAPISettings::$prefix . 'bexio_order_number');
        $order->delete_meta_data(AdminAPISettings::$prefix . 'bexio_invoice_id');
        $order->delete_meta_data(AdminAPISettings::$prefix . 'bexio_invoice_number');
        $order->delete_meta_data(AdminAPISettings::$prefix . 'bexio_delivery_id');
        $order->delete_meta_data(AdminAPISettings::$prefix . 'bexio_delivery_number');
        $order->save_meta_data();

        return new \WP_REST_Response(["success" => true]);
    }

    /**
     * Permission Callback
     **/
    public function get_permission()
    {
        if (current_user_can('administrator') || current_user_can('manage_woocommerce')) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Cloning is forbidden.
     */
    public function __clone()
    {
        _doing_it_wrong(__FUNCTION__, __('Cheatin&#8217; huh?'), BEXIO_CONNECTOR_BY_HELVY_VERSION);
    }
}
